// Copyright (C) 2020   John Donoghue   <john.donoghue@ieee.org>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.

#include <octave/oct.h>

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#ifdef BUILD_USBTMC
#include "usbtmc_class.h"
#endif

// PKG_ADD: autoload ("usbtmc_timeout", "usbtmc.oct");
DEFUN_DLD (usbtmc_timeout, args, nargout,
        "-*- texinfo -*-\n\
@deftypefn {Loadable Function} {@var{timeout} = } usbtmc_timeout(@var{usbtmc})\n \
@deftypefnx {Loadable Function} {@var{retcode} = } usbtmc_timeout(@var{usbtmc}, @var{timeout})\n \
\n\
Get or set the usbtmc timeout value\n \
\n\
@subsubheading Inputs\n \
@var{usbtmc} - instance of @var{octave_usbtmc} class.@* \
@var{timeout} - timeout value in milliseconds to set\n \
\n\
@subsubheading Outputs\n \
@var{retcode} - Return code from setting the timeout.@*\n \
@var{timeout} - timeout value in milliseconds.\n \
@end deftypefn")
{
#ifndef BUILD_USBTMC
  error ("usbtmc: Your system doesn't support the USBTMC interface");
  return octave_value ();
#else

  if (args.length () < 1 || args.length () > 2 || args (0).type_id () != octave_usbtmc::static_type_id ())
    {
      print_usage ();
      return octave_value (-1);
    }

  octave_usbtmc* usbtmc = NULL;

  const octave_base_value& rep = args (0).get_rep ();
  usbtmc = &((octave_usbtmc &)rep);

  // set time
  if (args.length () > 1)
    {
      if (! (args (1).OV_ISINTEGER () || args (1).OV_ISFLOAT ()) )
        {
          print_usage ();
          return octave_value (-1);
        }
      double timeout = args(1).double_value();

      if (timeout < 0 || timeout > 100000)
        {
          error ("Invalid range for timeout");
          return octave_value (-1);
	}

      int retval = usbtmc->set_timeout ((unsigned long)timeout);

      return octave_value(retval); 
    }
  else
    {
      unsigned long timeout = usbtmc->get_timeout();
      return octave_value((double)timeout);
    }
#endif
}
